"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const common_1 = require("./common");
class ValidationError extends Error {
    constructor() {
        super();
        this.message = 'Validate TypeScript';
    }
    get trace() { return this.reason(); }
    indent(count, message) {
        return ('\n' + message).replace(/\n/g, '\n' + '  '.repeat(count));
    }
    ;
}
exports.ValidationError = ValidationError;
class ConversionError extends ValidationError {
    constructor(value, converter, details) {
        super();
        this.value = value;
        this.converter = converter;
        this.details = details;
    }
    reason(count = 0) {
        return this.indent(count, `${chalk_1.default.yellowBright(`[Conversion: ${this.converter}]`)}\n${chalk_1.default.bold(`${common_1.objectToString(this.value)} ${this.details}.`)}`) + '\n';
    }
}
exports.ConversionError = ConversionError;
class AssertionError extends ValidationError {
    constructor(value, assertion, details) {
        super();
        this.value = value;
        this.assertion = assertion;
        this.details = details;
    }
    reason(count = 0) {
        return this.indent(count, `${chalk_1.default.greenBright(`[Assertion: ${this.assertion}]`)}\n${chalk_1.default.bold(`${common_1.objectToString(this.value)} ${this.details}.`)}`) + '\n';
    }
}
exports.AssertionError = AssertionError;
class ValidatorError extends ValidationError {
    constructor(validator, property, value, child_error) {
        super();
        this.validator = validator;
        this.property = property;
        this.value = value;
        this.child_error = child_error;
    }
    reason(count = 0) {
        let response = this.indent(count, `${chalk_1.default.blueBright(`[Validator: ${this.validator}]`)} on property ${chalk_1.default.redBright(`${this.property}`)}`);
        if (this.child_error instanceof ValidationError) {
            response += this.child_error.reason(count + 1);
        }
        return response;
    }
}
exports.ValidatorError = ValidatorError;
class NotMatchAnyError extends ValidationError {
    constructor(value, child_errors = []) {
        super();
        this.value = value;
        this.child_errors = child_errors;
    }
    reason(count = 0) {
        let response = '';
        // Hide options if there is only one.
        if (this.child_errors.length > 1)
            response = '\n' + this.indent(count, `${chalk_1.default.magentaBright(`[Options: ${this.child_errors.length}]`)}`) + '\n';
        else
            count--;
        for (let child_error of this.child_errors) {
            if (child_error instanceof ValidationError) {
                response += child_error.reason(count + 1);
            }
        }
        return response;
    }
}
exports.NotMatchAnyError = NotMatchAnyError;
//# sourceMappingURL=errors.js.map